#include "file_pager.h"
#include "file_paging.h"

#include <systypes/fcntl.h>



FilePaging::FilePaging(Pages *pages)
: _pages(pages)
{
}



/* Return any registered page mapper for the given 'fileid' or NULL if no such
   mapper is registered. */

PageMapper *FilePaging::get(fileid_t fileid)
{
    FileMapping::iterator entry = _mappers.find(fileid);
    PageMapper *mapper;

    if (entry == _mappers.end())
        mapper = NULL;
    else
        mapper = entry->second;

    return mapper;
}

/* Register a page 'mapper' for the given 'fileid'. */

void FilePaging::set(fileid_t fileid, PageMapper *mapper)
{
    FileMapping::iterator entry = _mappers.find(fileid);

    if (entry == _mappers.end())
        _mappers[fileid] = mapper;
}



/* Convert opening flags to map-compatible paging flags. */

flags_t FilePaging::get_flags(flags_t flags)
{
    return flags & (O_WRONLY | O_RDWR) ? L4RE_DS_MAP_FLAG_RW : L4RE_DS_MAP_FLAG_RO;
}



/* Obtain a page mapper for the 'fileid' or register a new one in the
   paging object. */

PageMapper *FilePaging::get_mapper(fileid_t fileid)
{
    /* Obtain any registered page mapper. */

    PageMapper *mapper = get(fileid);

    if (mapper != NULL)
        return mapper;

    /* Make an accessor and page mapper, registering the mapper. */

    Accessor *accessor = make_accessor(fileid);
    mapper = new PageMapper(accessor, _pages);

    set(fileid, mapper);

    return mapper;
}



/* Return a pager initialised with a page mapper. */

Pager *FilePaging::get_pager(fileid_t fileid, flags_t flags)
{
    std::lock_guard<std::mutex> guard(_lock);

    /* Initialise the pager with the mapper and a reference to this object for
       closing the mapper and accessor. */

    PageMapper *mapper = get_mapper(fileid);
    return new FilePager(fileid, mapper, flags, this);
}

/* Detach a pager, potentially removing its resources. */

void FilePaging::detach_pager(fileid_t fileid, PageMapper *mapper)
{
    std::lock_guard<std::mutex> guard(_lock);

    if (!mapper->detach())
    {
        _mappers.erase(fileid);
        delete mapper->accessor();
        delete mapper;
    }
}

// vim: tabstop=4 expandtab shiftwidth=4
